@@ -1,4 +1,22 @@
 #!/usr/bin/env python
+#  SSSD
+#
+#  SSSD Config API tests
+#
+#  Copyright (C) Red Hat
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  You should have received a copy of the GNU General Public License
+#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 '''
 Created on Sep 18, 2009
 
@@ -67,36 +85,36 @@ class SSSDConfigTestValid(unittest.TestCase):
         new_options = new_sssd_service.list_options();
 
         self.assertTrue('debug_level' in new_options)
-        self.assertEquals(new_options['debug_level'][0], int)
+        self.assertEqual(new_options['debug_level'][0], int)
 
         self.assertTrue('command' in new_options)
-        self.assertEquals(new_options['command'][0], str)
+        self.assertEqual(new_options['command'][0], str)
 
         self.assertTrue('reconnection_retries' in new_options)
-        self.assertEquals(new_options['reconnection_retries'][0], int)
+        self.assertEqual(new_options['reconnection_retries'][0], int)
 
         self.assertTrue('services' in new_options)
-        self.assertEquals(new_options['debug_level'][0], int)
+        self.assertEqual(new_options['debug_level'][0], int)
 
         self.assertTrue('domains' in new_options)
-        self.assertEquals(new_options['domains'][0], list)
-        self.assertEquals(new_options['domains'][1], str)
+        self.assertEqual(new_options['domains'][0], list)
+        self.assertEqual(new_options['domains'][1], str)
 
         self.assertTrue('sbus_timeout' in new_options)
-        self.assertEquals(new_options['sbus_timeout'][0], int)
+        self.assertEqual(new_options['sbus_timeout'][0], int)
 
         self.assertTrue('re_expression' in new_options)
-        self.assertEquals(new_options['re_expression'][0], str)
+        self.assertEqual(new_options['re_expression'][0], str)
 
         self.assertTrue('full_name_format' in new_options)
-        self.assertEquals(new_options['full_name_format'][0], str)
+        self.assertEqual(new_options['full_name_format'][0], str)
 
         self.assertTrue('default_domain_suffix' in new_options)
-        self.assertEquals(new_options['default_domain_suffix'][0], str)
+        self.assertEqual(new_options['default_domain_suffix'][0], str)
 
         self.assertTrue('domain_resolution_order' in new_options)
-        self.assertEquals(new_options['domain_resolution_order'][0], list)
-        self.assertEquals(new_options['domain_resolution_order'][1], str)
+        self.assertEqual(new_options['domain_resolution_order'][0], list)
+        self.assertEqual(new_options['domain_resolution_order'][1], str)
 
         del sssdconfig
 
@@ -573,7 +591,6 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'enumerate',
             'cache_credentials',
             'cache_credentials_minimal_first_factor_length',
-            'store_legacy_passwords',
             'use_fully_qualified_names',
             'ignore_group_members',
             'filter_users',
@@ -944,7 +961,6 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
             'enumerate',
             'cache_credentials',
             'cache_credentials_minimal_first_factor_length',
-            'store_legacy_passwords',
             'use_fully_qualified_names',
             'ignore_group_members',
             'filter_users',
@@ -1111,15 +1127,15 @@ class SSSDConfigTestSSSDDomain(unittest.TestCase):
         domain.set_option('krb5_realm', 'EXAMPLE.COM')
         domain.set_option('ldap_uri', 'ldap://ldap.example.com')
 
-        self.assertEquals(domain.get_option('krb5_realm'),
-                          'EXAMPLE.COM')
-        self.assertEquals(domain.get_option('ldap_uri'),
-                          'ldap://ldap.example.com')
+        self.assertEqual(domain.get_option('krb5_realm'),
+                         'EXAMPLE.COM')
+        self.assertEqual(domain.get_option('ldap_uri'),
+                         'ldap://ldap.example.com')
 
         # Remove the LDAP provider and verify that krb5_realm remains
         domain.remove_provider('id')
-        self.assertEquals(domain.get_option('krb5_realm'),
-                  'EXAMPLE.COM')
+        self.assertEqual(domain.get_option('krb5_realm'),
+                         'EXAMPLE.COM')
         self.assertFalse('ldap_uri' in domain.options)
 
         # Put the LOCAL provider back
@@ -1566,7 +1582,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         # First need to remove the existing service
         sssdconfig.delete_service('sssd')
         service = sssdconfig.new_service('sssd')
-        self.failUnless(service.get_name() in sssdconfig.list_services())
+        self.assertTrue(service.get_name() in sssdconfig.list_services())
 
         # TODO: check that the values of this new service
         # are set to the defaults from the schema
@@ -1836,8 +1852,8 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         # Positive Test
         domain = sssdconfig.new_domain('example.com')
         self.assertTrue(isinstance(domain, SSSDConfig.SSSDDomain))
-        self.failUnless(domain.get_name() in sssdconfig.list_domains())
-        self.failUnless(domain.get_name() in sssdconfig.list_inactive_domains())
+        self.assertTrue(domain.get_name() in sssdconfig.list_domains())
+        self.assertTrue(domain.get_name() in sssdconfig.list_inactive_domains())
 
         # TODO: check that the values of this new domain
         # are set to the defaults from the schema
@@ -1898,7 +1914,7 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertFalse('example.com' in sssdconfig.list_active_domains())
         self.assertFalse('example.com' in sssdconfig.list_inactive_domains())
         self.assertFalse(sssdconfig.has_section('domain/example.com'))
-        self.assertEquals(domain.oldname, None)
+        self.assertEqual(domain.oldname, None)
 
         # Positive test - Set the domain inactive and save it
         activelist = sssdconfig.list_active_domains()
@@ -1910,10 +1926,10 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertFalse('example.com2' in sssdconfig.list_active_domains())
         self.assertTrue('example.com2' in sssdconfig.list_inactive_domains())
 
-        self.assertEquals(len(sssdconfig.list_active_domains()),
-                          len(activelist)-1)
-        self.assertEquals(len(sssdconfig.list_inactive_domains()),
-                          len(inactivelist)+1)
+        self.assertEqual(len(sssdconfig.list_active_domains()),
+                         len(activelist)-1)
+        self.assertEqual(len(sssdconfig.list_inactive_domains()),
+                         len(inactivelist)+1)
 
         # Positive test - Set the domain active and save it
         activelist = sssdconfig.list_active_domains()
@@ -1924,10 +1940,10 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertTrue('example.com2' in sssdconfig.list_active_domains())
         self.assertFalse('example.com2' in sssdconfig.list_inactive_domains())
 
-        self.assertEquals(len(sssdconfig.list_active_domains()),
-                          len(activelist)+1)
-        self.assertEquals(len(sssdconfig.list_inactive_domains()),
-                          len(inactivelist)-1)
+        self.assertEqual(len(sssdconfig.list_active_domains()),
+                         len(activelist)+1)
+        self.assertEqual(len(sssdconfig.list_inactive_domains()),
+                         len(inactivelist)-1)
 
         # Positive test - Set the domain inactive and save it
         activelist = sssdconfig.list_active_domains()
@@ -1938,10 +1954,10 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertFalse('example.com2' in sssdconfig.list_active_domains())
         self.assertTrue('example.com2' in sssdconfig.list_inactive_domains())
 
-        self.assertEquals(len(sssdconfig.list_active_domains()),
-                          len(activelist)-1)
-        self.assertEquals(len(sssdconfig.list_inactive_domains()),
-                          len(inactivelist)+1)
+        self.assertEqual(len(sssdconfig.list_active_domains()),
+                         len(activelist)-1)
+        self.assertEqual(len(sssdconfig.list_inactive_domains()),
+                         len(inactivelist)+1)
 
         # Positive test - Set the domain active and save it
         activelist = sssdconfig.list_active_domains()
@@ -1952,10 +1968,10 @@ class SSSDConfigTestSSSDConfig(unittest.TestCase):
         self.assertTrue('example.com2' in sssdconfig.list_active_domains())
         self.assertFalse('example.com2' in sssdconfig.list_inactive_domains())
 
-        self.assertEquals(len(sssdconfig.list_active_domains()),
-                          len(activelist)+1)
-        self.assertEquals(len(sssdconfig.list_inactive_domains()),
-                          len(inactivelist)-1)
+        self.assertEqual(len(sssdconfig.list_active_domains()),
+                         len(activelist)+1)
+        self.assertEqual(len(sssdconfig.list_inactive_domains()),
+                         len(inactivelist)-1)
 
         # Positive test - Ensure that saved domains retain values
         domain.set_option('ldap_krb5_init_creds', True)
