@@ -33,8 +33,7 @@
 #include <ldb.h>
 #include <dhash.h>
 
-#include "data_provider/rdp.h"
-#include "sbus/sssd_dbus.h"
+#include "sss_iface/sss_iface_async.h"
 #include "responder/common/negcache.h"
 #include "sss_client/sss_cli.h"
 #include "responder/common/cache_req/cache_req_domain.h"
@@ -56,12 +55,12 @@ extern hash_table_t *dp_requests;
  */
 #define NEED_CHECK_PROVIDER(provider) \
     (provider != NULL && \
-     (strcmp(provider, "local") != 0 && \
+     ((!local_provider_is_built() || strcmp(provider, "local") != 0) && \
       strcmp(provider, "files") != 0))
 
 #define NEED_CHECK_AUTH_PROVIDER(provider) \
     (provider != NULL && \
-      strcmp(provider, "local") != 0)
+      (!local_provider_is_built() || strcmp(provider, "local") != 0))
 
 /* needed until nsssrv.h is updated */
 struct cli_request {
@@ -95,6 +94,7 @@ struct be_conn {
     const char *cli_name;
     struct sss_domain_info *domain;
 
+    char *bus_name;
     char *sbus_address;
     struct sbus_connection *conn;
 };
@@ -130,8 +130,6 @@ struct resp_ctx {
     const char *sss_pipe_name;
     const char *confdb_service_path;
 
-    hash_table_t *dp_request_table;
-
     struct timeval get_domains_last_call;
 
     size_t allowed_uids_count;
@@ -211,11 +209,8 @@ int sss_process_init(TALLOC_CTX *mem_ctx,
                      const char *sss_priv_pipe_name,
                      int priv_pipe_fd,
                      const char *confdb_service_path,
+                     const char *conn_name,
                      const char *svc_name,
-                     uint16_t svc_version,
-                     struct mon_cli_iface *monitor_intf,
-                     const char *cli_name,
-                     struct sbus_iface_map *sbus_iface,
                      connection_setup_t conn_setup,
                      struct resp_ctx **responder_ctx);
 
@@ -257,59 +252,12 @@ sss_cmd_check_cache(struct ldb_message *msg,
                     int cache_refresh_percent,
                     uint64_t cache_expire);
 
-typedef void (*sss_dp_callback_t)(uint16_t err_maj, uint32_t err_min,
-                                  const char *err_msg, void *ptr);
-
-struct dp_callback_ctx {
-    sss_dp_callback_t callback;
-    void *ptr;
-
-    void *mem_ctx;
-    struct cli_ctx *cctx;
-};
-
 void handle_requests_after_reconnect(struct resp_ctx *rctx);
 
-int responder_logrotate(struct sbus_request *dbus_req, void *data);
-
-/* Each responder-specific request must create a constructor
- * function that creates a DBus Message that would be sent to
- * the back end
- */
-typedef DBusMessage * (dbus_msg_constructor)(void *);
-
-/*
- * This function is indended for consumption by responders to create
- * responder-specific requests such as sss_dp_get_account_send for
- * downloading account data.
- *
- * Issues a new back end request based on strkey if not already running
- * or registers a callback that is called when an existing request finishes.
- */
 errno_t
-sss_dp_issue_request(TALLOC_CTX *mem_ctx, struct resp_ctx *rctx,
-                     const char *strkey, struct sss_domain_info *dom,
-                     dbus_msg_constructor msg_create, void *pvt,
-                     struct tevent_req *nreq);
-
-/* Every provider specific request uses this structure as the tevent_req
- * "state" structure.
- */
-struct sss_dp_req_state {
-    dbus_uint16_t dp_err;
-    dbus_uint32_t dp_ret;
-    char *err_msg;
-};
-
-/* The _recv functions of provider specific requests usually need to
- * only call sss_dp_req_recv() to get return codes from back end
- */
-errno_t
-sss_dp_req_recv(TALLOC_CTX *mem_ctx,
-                struct tevent_req *sidereq,
-                dbus_uint16_t *dp_err,
-                dbus_uint32_t *dp_ret,
-                char **err_msg);
+responder_logrotate(TALLOC_CTX *mem_ctx,
+                    struct sbus_request *sbus_req,
+                    struct resp_ctx *rctx);
 
 /* Send a request to the data provider
  * Once this function is called, the communication
@@ -344,24 +292,9 @@ sss_dp_get_account_send(TALLOC_CTX *mem_ctx,
 errno_t
 sss_dp_get_account_recv(TALLOC_CTX *mem_ctx,
                         struct tevent_req *req,
-                        dbus_uint16_t *err_maj,
-                        dbus_uint32_t *err_min,
-                        char **err_msg);
-
-struct tevent_req *
-sss_dp_get_ssh_host_send(TALLOC_CTX *mem_ctx,
-                         struct resp_ctx *rctx,
-                         struct sss_domain_info *dom,
-                         bool fast_reply,
-                         const char *name,
-                         const char *alias);
-
-errno_t
-sss_dp_get_ssh_host_recv(TALLOC_CTX *mem_ctx,
-                         struct tevent_req *req,
-                         dbus_uint16_t *dp_err,
-                         dbus_uint32_t *dp_ret,
-                         char **err_msg);
+                        uint16_t *_dp_error,
+                        uint32_t *_error,
+                        const char **_error_message);
 
 bool sss_utf8_check(const uint8_t *s, size_t n);
 
@@ -489,4 +422,17 @@ int resp_resolve_group_names_recv(TALLOC_CTX *mem_ctx,
                                   struct tevent_req *req,
                                   struct ldb_result **_initgr_named_res);
 
+/**
+ * Register common responder sbus interface on connection.
+ */
+errno_t
+sss_resp_register_sbus_iface(struct sbus_connection *conn,
+                             struct resp_ctx *rctx);
+
+/**
+ * Register common service sbus interface on monitor connection.
+ */
+errno_t
+sss_resp_register_service_iface(struct resp_ctx *rctx);
+
 #endif /* __SSS_RESPONDER_H__ */
