@@ -47,7 +47,8 @@
 #include "util/debug.h"
 
 /* name of the monitor server instance */
-#define SSSD_PIDFILE PID_PATH"/sssd.pid"
+#define SSSD_MONITOR_NAME        "sssd"
+#define SSSD_PIDFILE PID_PATH"/"SSSD_MONITOR_NAME".pid"
 #define MAX_PID_LENGTH 10
 
 #define _(STRING) gettext (STRING)
@@ -66,8 +67,6 @@
 #define NULL 0
 #endif
 
-#define ZERO_STRUCT(x) memset((char *)&(x), 0, sizeof(x))
-
 #define SSSD_MAIN_OPTS SSSD_DEBUG_OPTS
 
 #define SSSD_SERVER_OPTS(uid, gid) \
@@ -176,7 +175,7 @@ struct main_context {
 errno_t server_common_rotate_logs(struct confdb_ctx *confdb,
                                   const char *conf_entry);
 int die_if_parent_died(void);
-int pidfile(const char *path, const char *name);
+int pidfile(const char *file);
 int server_setup(const char *name, int flags,
                  uid_t uid, gid_t gid,
                  const char *conf_entry,
@@ -371,6 +370,7 @@ struct cert_verify_opts {
     bool do_verification;
     char *ocsp_default_responder;
     char *ocsp_default_responder_signing_cert;
+    char *crl_file;
 };
 
 errno_t parse_cert_verify_opts(TALLOC_CTX *mem_ctx, const char *verify_opts,
@@ -399,14 +399,17 @@ bool is_socket_activated(void);
 /* Returns true if the responder has been dbus-activated */
 bool is_dbus_activated(void);
 
+/* Returns true if SSSD was built with local provider support */
+bool local_provider_is_built(void);
+
 /**
  * @brief Add two list of strings
  *
- * Create a new NULL-termintated list of strings by adding two lists together.
+ * Create a new NULL-terminated list of strings by adding two lists together.
  *
  * @param[in] mem_ctx      Talloc memory context for the new list.
- * @param[in] l1           First NULL-termintated list of strings.
- * @param[in] l2           Second NULL-termintated list of strings.
+ * @param[in] l1           First NULL-terminated list of strings.
+ * @param[in] l2           Second NULL-terminated list of strings.
  * @param[in] copy_strings If set to 'true' the list items will be copied
  *                         otherwise only the pointers to the items are
  *                         copied.
@@ -557,6 +560,9 @@ find_domain_by_object_name_ex(struct sss_domain_info *domain,
 bool subdomain_enumerates(struct sss_domain_info *parent,
                           const char *sd_name);
 
+char *subdomain_create_conf_path_from_str(TALLOC_CTX *mem_ctx,
+                                          const char *parent_name,
+                                          const char *subdom_name);
 char *subdomain_create_conf_path(TALLOC_CTX *mem_ctx,
                                  struct sss_domain_info *subdomain);
 
@@ -571,6 +577,12 @@ void sss_domain_info_set_output_fqnames(struct sss_domain_info *domain,
 
 bool sss_domain_info_get_output_fqnames(struct sss_domain_info *domain);
 
+bool sss_domain_is_mpg(struct sss_domain_info *domain);
+
+enum sss_domain_mpg_mode get_domain_mpg_mode(struct sss_domain_info *domain);
+const char *str_domain_mpg_mode(enum sss_domain_mpg_mode mpg_mode);
+enum sss_domain_mpg_mode str_to_domain_mpg_mode(const char *str_mpg_mode);
+
 #define IS_SUBDOMAIN(dom) ((dom)->parent != NULL)
 
 #define DOM_HAS_VIEWS(dom) ((dom)->has_views)
@@ -660,6 +672,7 @@ int sss_del_seuser(const char *login_name);
 int sss_get_seuser(const char *linuxuser,
                    char **selinuxuser,
                    char **level);
+int sss_seuser_exists(const char *linuxuser);
 
 /* convert time from generalized form to unix time */
 errno_t sss_utc_to_time_t(const char *str, const char *format, time_t *unix_time);
@@ -720,6 +733,11 @@ errno_t create_preauth_indicator(void);
 #define P11_CHILD_LOG_FILE "p11_child"
 #define P11_CHILD_PATH SSSD_LIBEXEC_PATH"/p11_child"
 #define P11_CHILD_TIMEOUT_DEFAULT 10
+#define P11_WAIT_FOR_CARD_TIMEOUT_DEFAULT 60
 #endif  /* SSSD_LIBEXEC_PATH */
 
+#ifndef N_ELEMENTS
+#define N_ELEMENTS(arr) (sizeof(arr) / sizeof(arr[0]))
+#endif
+
 #endif /* __SSSD_UTIL_H__ */
